using Assets;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Enemy : MonoBehaviour
{
    Rigidbody2D rb;
    //____________________
    private Player player;
   
    public int colDmg = 1;

    public float knockback;
    public float knockbackLength;
    public float knockbackCount;
    public bool knockFromRight;

    //______________________________________________________
    [SerializeField]
    Transform CastPoint1;
    
    [SerializeField]
    Transform CastPoint2;

    [SerializeField]
    GameObject golem;

    Animator golemAnimator;

    [SerializeField]
    float hitRange = 5;

    [SerializeField]
    float aggroRange;

    [SerializeField]
    float aggroHeight;

    [SerializeField]
    float aggroTime;

    [SerializeField]
    float moveSpeed;

    public bool isFacingLeft;
    public bool isChasing = false;
    public bool waitingToStopChasing = false;
    public bool isSearching = false;
    public bool isAttacking = false;

    public float searchTimeElaped = 0;
    public float timeFromLastAttack = 0;


    AudioSource audioSource;
    public AudioClip swingAudio;
    //______________________________________________________

    void Start()
    {
        rb = GetComponent<Rigidbody2D>();
        player = GameObject.FindGameObjectWithTag("Player").GetComponent<Player>();
        //______________________________________________________
        
        golemAnimator = golem.GetComponent<Animator>();
        audioSource = GetComponent<AudioSource>();
        //______________________________________________________
    }

    public IEnumerator Knockback(float knockDur, float knockPwr, Vector3 knockDir)
    {
        float timer = 0;

        while (knockDur > timer)
        {
            timer += Time.deltaTime;
            rb.AddForce(new Vector3((knockDir.x * -100 * transform.localScale.x), knockDir.y + knockPwr, transform.position.z));
        }
        yield return 0;
    }

    public void PlaySwingAudioClip()
    {
        if(audioSource && swingAudio)
        {
            audioSource.PlayOneShot(swingAudio);
        }
    }


   
    void Update()
    {
        if (CanSeePlayer(aggroRange))
        {
            if(!isChasing)
            {
                isChasing = true;
                waitingToStopChasing = false;
                CancelInvoke("StopChasing");
            }
            
        }
        else
        {
            if (isChasing && !waitingToStopChasing)
            {
                waitingToStopChasing = true;
                Invoke("StopChasing", aggroTime);
            }
        }

        if (isChasing)
        {
            ChasePlayer();
        }

        golemAnimator.SetBool("isAttacking", isAttacking);
        golemAnimator.SetFloat("moveSpeed", Mathf.Abs(rb.velocity.x));

        if (isChasing || isSearching || Mathf.Abs(rb.velocity.y) > 1)
        {
            rb.constraints = RigidbodyConstraints2D.FreezeRotation;
        } else
        {
            rb.constraints = RigidbodyConstraints2D.FreezeRotation | RigidbodyConstraints2D.FreezePositionX;
        }
    }

    void StopChasing()
    {
        isChasing = false;
        waitingToStopChasing = false;
    }

    void ChasePlayer()
    {
        if (Vector2.Distance(transform.position, player.transform.position) < hitRange)
        {
            //attack
            isAttacking = true;
            rb.velocity = new Vector2(0, rb.velocity.y);

            if (transform.position.x < player.transform.position.x)
            {
                isFacingLeft = false;
            } else
            {
                isFacingLeft = true;
            }

            return;
        }

        isAttacking = false;
        timeFromLastAttack = 0;

        if (transform.position.x < player.transform.position.x)
        {
            // enemy to the left side of player, move right
            rb.velocity = new Vector2(moveSpeed, rb.velocity.y);
            transform.localScale = new Vector2(-1, 1);
            isFacingLeft = false;
        }
        else
        {
            // enemy to the right side of player, move left
            rb.velocity = new Vector2(-moveSpeed, rb.velocity.y);
            transform.localScale = new Vector2(1, 1);
            isFacingLeft = true;
        }
    }

    void DealDamageToPlayer()
    {
        HealthManager hm = player.GetComponent<HealthManager>();
        if (hm.hp > 0)
        {
            
            hm.TakeDamage(colDmg);
        }
       
    }

    void SearchForPlayer()
    {

    }

    bool CanSeePlayer(float distance)
    {
        bool val = false;


        Vector2 size = new Vector2(distance * 2, aggroHeight);
        RaycastHit2D hit1 = Physics2D.BoxCast(transform.position, size, 0, Vector2.left, 0, LayerMask.GetMask("Player"));

        if (hit1.collider != null)
        {
            if (hit1.collider.gameObject.CompareTag("Player"))
            {
                val = true;
            }
            else
            {
                val = false;
            }

            Debug.DrawLine(transform.position, hit1.point, Color.red);
        }
        else
        {

        }

        Utils.DrawBox2D(new Vector2(transform.position.x, transform.position.y), size, Color.yellow);

        return val;
    }
}
