using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class HealthManager : MonoBehaviour
{
    [SerializeField]
    public int hp = 10;

    public int maxHp { get; } = 10;

    public GameObject deathEffect;
    public GameObject impactEffect;
    public AudioClip onHitAudio;
    public AudioClip onHealthCollectAudio;
    public float disposeTime = 0.2f;
    private Player player;
    Rigidbody2D RigidBody;
    AudioSource AudioSource;

    public AnimationClip DamageAnimation;

    public bool isDead = false;

    void Start()
    {
        RigidBody = GetComponent<Rigidbody2D>();
        AudioSource = GetComponent<AudioSource>();
        player = GameObject.FindGameObjectWithTag("Player").GetComponent<Player>();
        //set default checkpoint;
        if (gameObject.CompareTag("Player"))
        {
            GameObject managerObject = GameObject.FindGameObjectWithTag("RespawnManager");
            RespawnManager respawnManager = managerObject.GetComponent<RespawnManager>();
            respawnManager.SetLastCheckpoint(gameObject.transform.position);
        }
    }


    void Update()
    {
        
    }

    public void TakeDamage(int damage)
    {

        

        hp -= damage;
        if (hp <= 0)
        {
            Die();
        } else
        {
            Animation anim = gameObject.GetComponent<Animation>();
            if (anim != null && DamageAnimation != null)
            {
                anim.clip = DamageAnimation;
                anim.Play();
            }

            if (onHitAudio != null && AudioSource != null)
            {
                AudioSource.PlayOneShot(onHitAudio);
            }
        }
    }

    void Die()
    {
        if (isDead) return;
        isDead = true;

        GameObject Effect = Instantiate(deathEffect, transform.position, Quaternion.identity);
        Destroy(Effect, disposeTime);

        if (gameObject.CompareTag("Player"))
        {

            Player.score -= 100;
            if(Player.score < 0)
            {
                Player.score = 0;
            }

            GameObject managerObject = GameObject.FindGameObjectWithTag("RespawnManager");
            if(managerObject != null)
            {
                RespawnManager respawnManager = managerObject.GetComponent<RespawnManager>();
                respawnManager.RespawnPlayer(gameObject);
                gameObject.SetActive(false);
            }
        } else
        {
            Destroy(gameObject);    
        }
    }

    public void KnockBack(Vector2 Amount, bool absoluteX = false, bool absoluteY = false)
    {
        if(RigidBody != null)
        {
            if(absoluteX)
            {
                RigidBody.velocity = new Vector2(0, RigidBody.velocity.y);
            }

            if (absoluteY)
            {
                RigidBody.velocity = new Vector2(RigidBody.velocity.x, 0);
            }

            RigidBody.velocity += Amount;
        }

    }

    public bool Heal(int amount)
    {
        if(hp < maxHp)
        {
            hp += amount;

            if (AudioSource && onHealthCollectAudio)
            {
                AudioSource.PlayOneShot(onHealthCollectAudio);
            }
            return true;
        }

        return false;
    }


}
