using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class ObjectActivator : MonoBehaviour
{
    public float distance;
    public bool drawDebug = false;
    List<ObjectActivatorRegister> registeredObjects = new List<ObjectActivatorRegister>();

    void Start()
    {
    }

    void Update()
    {
        foreach(var registeredObject in registeredObjects) {
            CheckObjectActiveState(registeredObject);

            if(drawDebug && registeredObject.enableCustomDistance)
            {
                Vector2 objectPosition = registeredObject.transform.position;
                float customDistance = registeredObject.customDistance;
                Debug.DrawLine(objectPosition, objectPosition + Vector2.right * customDistance, Color.magenta);
                Debug.DrawLine(objectPosition, objectPosition + Vector2.right * customDistance * -1, Color.magenta);
                Debug.DrawLine(objectPosition, objectPosition + Vector2.up * customDistance, Color.magenta);
                Debug.DrawLine(objectPosition, objectPosition + Vector2.up * customDistance * -1, Color.magenta);
            }
        }

        if(drawDebug)
        {
            Vector2 objectPosition = gameObject.transform.position;
            Debug.DrawLine(objectPosition, objectPosition + Vector2.right * distance, Color.magenta);
            Debug.DrawLine(objectPosition, objectPosition + Vector2.right * distance * -1, Color.magenta);
        }
    }

    public void Register(ObjectActivatorRegister inObject)
    {
        registeredObjects.Add(inObject);
        CheckObjectActiveState(inObject);
    }

    public void Unregister(ObjectActivatorRegister inObject)
    {
        registeredObjects.Remove(inObject);
    }

    void CheckObjectActiveState(ObjectActivatorRegister inObject)
    {
        if(inObject.unlimitedDistance)
        {
            if (!inObject.gameObject.activeSelf)
            {
                inObject.gameObject.SetActive(true);
            }
            return;
        }

        var checkDistance = inObject.enableCustomDistance ? inObject.customDistance : distance;

        if (Vector2.Distance(inObject.gameObject.transform.position, gameObject.transform.position) < checkDistance)
        {
            inObject.gameObject.SetActive(true);
        }
        else
        {
            inObject.gameObject.SetActive(false);
        }
    }
}
