using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class Player : MonoBehaviour
{
    Rigidbody2D rb;
    public Animator animator;
    [SerializeField]
    public bool useJoystick = true;
    private Joystick joystick;
    //strelba
    public Transform firePoint;
    public GameObject bulletPrefab;
    public int ammo = 5;
    private bool isShooting = false;
    private float  shotTimer = 0;
    [SerializeField]
    private float shotCD = 0.1f;

    private bool isAttacking = false;
    
    private float attackTimer = 0;
    [SerializeField]
    private float attackCD = 0.1f;

    static public int score = 0;
    
    //utok
    public AttackTrigger attackTrigger;

    //pohyb
    public CharacterController2D controller;
    public float runSpeed = 60f;
    float horizontalMove = 0f;
    bool jump = false;
    bool requestedJump = false;
    bool crouch = false;

    public AudioClip swingAudio;
    public AudioClip jumpAudio;
    public AudioClip pickAmmoSound;
    public AudioClip collectCoinAudio;
    AudioSource audioSource;


    //knockback
    public float knockback;
    public float knockbackLength;
    public float knockbackCount;


    void Start()
    {
        rb = GetComponent<Rigidbody2D>();
        joystick = GameObject.FindGameObjectWithTag("Joystick").GetComponent<Joystick>();

        audioSource = GetComponent<AudioSource>();
    }

    void FixedUpdate()
    {
       controller.Move(horizontalMove * Time.fixedDeltaTime, crouch, requestedJump);

       requestedJump = false;
    }

    static public void ResetScore()
    {
        score = 0;
        Debug.Log("Resetting score..");
    }

    void Update()
    {
        if (useJoystick)
        {
            float joystickAbs = Mathf.Abs(joystick.Horizontal);
            float speed = Mathf.Abs(joystick.Horizontal) < 0.55f ? (runSpeed * (joystickAbs + 0.15f)) : runSpeed;
            if (joystickAbs > 0.02f)
            {
                horizontalMove = Mathf.Sign(joystick.Horizontal) * speed;
            } else
            {
                horizontalMove = 0;
            }

            if(joystick.Vertical < -0.45f)
            {
                crouch = true;
            } else
            {
                crouch = false;
            }
        }
        else
        {
            horizontalMove = Input.GetAxisRaw("Horizontal") * runSpeed;

            if (Input.GetButtonDown("Crouch"))
            {
                crouch = true;

            }
            else if (Input.GetButtonUp("Crouch"))
            {

                if (!crouch)
                {

                    if (Physics2D.OverlapCircle(controller.m_CeilingCheck.position, 0.2f, controller.m_WhatIsGround))
                    {
                        crouch = true;

                    }
                    else
                    {
                        crouch = false;
                    }
                }
                

            }
            

        }

        jump = !CharacterController2D.m_Grounded;

        animator.SetBool("IsJumping", jump);
       
        animator.SetFloat("Speed", Mathf.Abs(horizontalMove));

        if (Input.GetButtonDown("Jump"))
        {
            Jump();
        }

        if (Input.GetButtonDown("Fire1"))
        {
            Shoot();
        }
       
        if (Input.GetButtonDown("Fire2") && !isAttacking && !crouch)
        {
            Attack();      
        }

        if (isAttacking)
        {
            if (attackTimer > 0)
            {
                attackTimer -= Time.deltaTime;
            }
            else
            {
                isAttacking = false;
                attackTrigger.DisableTrigger();
            }

            animator.SetBool("IsAttacking", isAttacking); 


        }
        
        if (isShooting)
        {

            if (shotTimer > 0)
            {
                shotTimer -= Time.deltaTime;
            }
            else
            {
                isShooting = false;
                
            }
            animator.SetBool("IsShooting", isShooting);
 
        }
    }

    public void OnCrouch(bool isCrouching)
    {
        animator.SetBool("IsCrouching", isCrouching);
    }
    void Awake()
    {
        animator = gameObject.GetComponent<Animator>();
        attackTrigger.DisableTrigger();
    }

    public void Jump()
    {
        requestedJump = true;
        if(CharacterController2D.m_Grounded && audioSource && jumpAudio)
        {
            audioSource.PlayOneShot(jumpAudio);
        }
    }


    public void Attack()
    {
        isAttacking = true;
        attackTimer = attackCD;

        attackTrigger.EnableTrigger();
        animator.SetBool("IsAttacking", isAttacking);

        if (audioSource && swingAudio)
        {
            audioSource.PlayOneShot(swingAudio);
        }
    }
   
    public void Shoot()
    {
        isShooting = true;
        shotTimer = shotCD;

        if (ammo > 0)
        {
            
            Instantiate(bulletPrefab, firePoint.position, firePoint.rotation);
            ammo = ammo - 1; 
            
            

        }
        else
        {
            
            Debug.Log("No ammo!");
        }
        
        animator.SetBool("IsShooting", isShooting);
        
    }
    public void AddScore(int amount)
    {
        score += amount;
        if(audioSource && collectCoinAudio)
        {
            audioSource.PlayOneShot(collectCoinAudio);
        }
        if (score < 0)
        {
            score = 0;
        }
    }


    public bool HasFullAmmo()
    {
        return ammo >= 5;
    }

    public void AddAmmo(int amount)
    {
        if (ammo < 5)
        {
            ammo = ammo + 1;
            if(audioSource && pickAmmoSound)
            {
                audioSource.PlayOneShot(pickAmmoSound);
            }
        }
    }
}
























   
